package File;

sub beforebegindocument { () }
sub afterbegindocument { () }
sub beforeenddocument { () }

sub new {
  my $proto = shift;
  my $class = ref($proto) || $proto;
  my $self  = {};
  $self->{TITLE} = shift;
  $self->{DATA} = [];
  foreach(@_){ push @{ $self->{DATA} }, UTF::any2utf($_); }
  $self->{TYPE} = undef;
  $self->{ENCODING} = UTF8;
  $self->{FILE} = ($ENV{TMPDIR} ? $ENV{TMPDIR} : "/tmp") ."/imprime$$";
  bless ($self, $class);
  return $self;
}

sub type {
  my $self = shift;
  my $type = shift;
  if( defined $type ){
    $self->{TYPE} = $type;
    bless $self, $type;
    print STDERR "  Type set to $type\n";
  } else {
    return $self->{TYPE};
  }
}

sub guess {
  my $self = shift;
  my $line;
  foreach (@{ $self->{DATA} }){
    $line = $_;
    last unless m/^\s*$/;
  }
  chomp($line);
  print STDERR "  Guessing type from line $line\n";
  if( $line =~ m/^(From|Path)/ ){ $self->type(MAIL) }
  elsif( $line =~ m/^\#\!.*perl/ ){ $self->type(PERL) }
  elsif( $line =~ m/(DOCTYPE\s*HTML|\<HTML)/ ){ $self->type(HTML) }
  else{ $self->type(UNKNOWN) }
}

sub transform {
  my $self = shift;
  # On retire les lignes blanches au dbut
#  shift @{ $self->{DATA} }
#    while( ${$self->{DATA}}[0] =~ m/^\s*$/ );
  # on retire les lignes blanches  la fin
#  pop @{ $self->{DATA} }
#    while( ${$self->{DATA}}[ $#{ $self->{DATA} } ] =~ m/^\s*$/ );
}

sub latex {
  my $self = shift;
  my $file = $self->{FILE};
  my $title = $self->{TITLE};
  open(LATEX, ">$file.tex") || die "Cannot open $file.tex for writing: $!";
  print LATEX '\documentclass[a4paper,12pt]{article}' ."\n";
  print LATEX '\usepackage[T1]{fontenc}' ."\n";
  print LATEX '%\usepackage[latin1]{inputenc}' ."\n";
  print LATEX '\usepackage{pslatex}' ."\n";
  print LATEX '\parindent=0pt' ."\n";
  print LATEX '\usepackage[a4paper,left=1.5cm,right=1.5cm,top=1.5cm,bottom=1.5cm]{geometry}' ."\n";
  print LATEX '\usepackage{fancyhdr}' ."\n";
  print LATEX '\pagestyle{fancy}' ."\n";
  print LATEX '\fancyhf{}' ."\n";
  print LATEX '\fancyfoot[RO,LE]{\textbf{\thepage}}' ."\n";
  print LATEX '\fancyfoot[LO,RE]{\textbf{\today}}' ."\n";
  print LATEX '\fancyhead[CO,CE]{\mbox{}'. $title .'}' ."\n";
  print LATEX '\renewcommand{\footrulewidth}{0pt}' ."\n";
  print LATEX '\renewcommand{\headrulewidth}{0pt}' ."\n";

  print LATEX '\DeclareFontFamily{T1}{song}{}
\DeclareFontShape{T1}{song}{m}{n}{<-> cyberb00}{}
\renewcommand\rmdefault{song}
\usepackage{CJK}
\usepackage{amsbsy}
\let\textbf\pmb
';

  foreach ($self->beforebegindocument()){ print LATEX "$_\n"; }
  print LATEX '\begin{document}' ."\n";
  print LATEX '\begin{CJK}{UTF8}{song}' ."\n";
  foreach ($self->afterbegindocument()){ print LATEX "$_\n"; }

  foreach ( @{ $self->{DATA} } ){
    print LATEX $_;
  }
  foreach ($self->beforeenddocument()){ print LATEX "$_\n"; }
  print LATEX '\end{CJK}' ."\n";
  print LATEX '\end{document}' ."\n";
  close LATEX;
}

sub dvi {
  my $self = shift;
  my $file = $self->{FILE};
  open(OLDOUT, ">&STDOUT");
  close STDOUT;
  system 'latex', '--interaction=nonstopmode', "$file.tex";
  open(STDOUT, ">&OLDOUT");
}

sub xdvi {
  my $self = shift;
  my $file = $self->{FILE};
  system 'xdvi', "$file.dvi";
}

sub ps {
  my $self = shift;
  my $file = $self->{FILE};
  system 'dvips', '-o', "$file.ps", "$file.dvi";
}

sub lpr {
  my $self = shift;
  my $file = $self->{FILE};

  my $nombre_de_pages = 0;
  open(PS, "$file.ps") || die "Cannot open $file.ps for reading: $!";
  while(<PS>) { $nombre_de_pages++ if /^\%\%Page\:/ ; }
  close(PS);
  if($nombre_de_pages < 2){
    system("lpr $file.ps");
  } else {
    system("psnup -pa4 -2 $file.ps | lpr");
  }
}

sub print {
  my $self = shift;
  my $extension = shift;
  my $file = $self->{FILE};
  open(F, "$file.$extension") || die "Cannot open $file.$extension: $!";
  while(<F>){ print }
  close F;
}

sub clean {
  my $self = shift;
  unlink $self->{FILE} .".raw";
  unlink $self->{FILE} .".tex";
  unlink $self->{FILE} .".log";
  unlink $self->{FILE} .".aux";
  unlink $self->{FILE} .".dvi";
  unlink $self->{FILE} .".ps";
}

sub verif {
  my $self = shift;
  my $file = $self->{FILE};

  open(F, ">$file.raw") || die "Cannot open $file.raw for writing: $!";
  foreach (@{ $self->{DATA} }){ print F $_; }
  close F;

  open(P, "file $file.raw|") || die "Cannot run `file $file.raw': $!";
  my $type = join('', <P>);
  close P;
  if( $type =~ m/data/ ){
    open(P, "identify $file.raw|") || die "Cannot run `identify $file.raw': $!";
    $type .= join('', <P>);
    close P;
  }

  ## PostScript File
  if( $type =~ m/postscript/i ){
    print STDERR "This seems to be a PostScript file\n";
    if( $Start::optctl{out} eq "xdvi" ){
      rename "$file.raw", "$file.ps";
      system 'gv', "$file.ps";
      $self->clean();
      exit 0;
    } elsif( $Start::optctl{out} eq "ps" ){
      $self->print("ps");
      $self->clean();
      exit 0;
    } elsif( $Start::optctl{out} eq "lpr" ){
      $self->lpr();
      $self->clean();
      exit 0;
    }

  ## DVI File
  } elsif( $type =~ m/TeX DVI/ ){
    print STDERR "This seems to be a DVI file\n";
    rename "$file.raw", "$file.dvi";
    if( $optctl{out} eq "dvi" ){
      $self->print("dvi");
      $self->clean;
      exit 0;
    }
    if( $optctl{out} eq "xdvi" ){
      $self->xdvi;
      $self->clean;
      exit 0;
    }
    $self->ps();
    if( $optctl{out} eq "ps" ){
      $self->print("ps");
      $self->clean;
      exit 0;
    }
    $self->lpr();
    $self->clean;
    exit 0;

  ## La(TeX) File
  } elsif( $type =~ m/TeX/ ){
    print STDERR "This seems to be a (La)TeX file: compile it\n";
    exit 1;

  ## PDF File
  } elsif( $type =~ m/PDF/i ){
    print STDERR "This seems to be a PDF file\n";
    rename "$file.raw", "$file.pdf";
    if( $Start::optctl{out} eq "xdvi" ){
      system 'acroread', "$file.pdf";
      $self->clean();
      exit 0;
    } elsif( $Start::optctl{out} eq "ps" ){
#      system 'pdf2ps', "$file.pdf", "$file.ps";
      system 'acroread', '-toPostScript', '-pairs', "$file.pdf", "$file.ps";
      $self->print("ps");
      $self->clean();
      exit 0;
    } elsif( $Start::optctl{out} eq "lpr" ){
      $self->lpr();
      $self->clean();
      exit 0;
    }

  ## JPEG/GIF File
  } elsif( $type =~ m/ (JPEG|GIF|TIFF|PNG) / ){
    print STDERR "This seems to be a Picture: do not print it.\n";
    exit 1;
  }
}
