package UTF;

use strict;
use Text::Iconv;
Text::Iconv->raise_error(0);
use MIME::Base64;
use MIME::QuotedPrint;

our %converters = (
  "EUC-JP"        => Text::Iconv->new("EUC-JP",        "UTF-8"),
  "SJIS"          => Text::Iconv->new("SJIS",          "UTF-8"),
  "UTF-8"         => Text::Iconv->new("UTF-8",         "UTF-8"),
  "ISO-2022-JP-2" => Text::Iconv->new("ISO-2022-JP-2", "UTF-8"),
);

sub guess {
  my $in = shift;

  my $euc  = defined $converters{"EUC-JP"}->convert($in);
  my $sjis = defined $converters{"SJIS"}->convert($in);
  my $utf  = defined $converters{"UTF-8"}->convert($in);
  my $iso  = defined $converters{"ISO-2022-JP-2"}->convert($in);

  return "UTF-8"  if $utf and not $euc and not $sjis and not $iso;
  return "EUC-JP"  if $euc and not $utf and not $sjis and not $iso;
  return "SJIS" if $sjis and not $euc and not $utf and not $iso;
  return "ISO-2022-JP-2" if $iso and not $euc and not $sjis and not $utf;

  # ISO-2022-JP-2 est le seul codage  contenir des ESC
  return "ISO-2022-JP-2" if $iso and $in =~ m/\e/;

  # Le texte en Unicode est le seul  contenir des couples de 0x8?/0x9?
  return "UTF-8" if $utf and $in =~ m/[\x80-\x9F][\x80-\x9F]/;

  # Le texte en SJIS a tendance  contenir des 0x82 et des 0x83 (hiragana et katakana)
  return "SJIS" if $sjis and $in =~ m/\x82|\x83/;

  # S'il y a des , c'est probablement du EUC
  return "EUC-JP" if $euc and $in =~ m//;
  # S'il y a des  suivis de quelque chose de non ascii, c'est probablement du EUC.
  # (Ca pourrait aussi etre de l'unicode, mais on s'en serait deja appercu)
  # (Ca ne devrait pas etre du latin1, car ce serait une faute de typographie)
  return "EUC-JP" if $euc and $in =~ m/[\x80-\xFF]/;

  # S'il y a des , c'est probablement de l'utfcode
  return "UTF-8" if $utf and $in =~ m//;

  # Sinon, on ne sait pas (probablement de l'ascii)
  return "ISO-2022-JP-2" if $iso;
  return "SJIS" if $sjis;
  return "EUC-JP" if $euc;
  return "UTF-8" if $utf;

  # On ne devrait pas arriver ici.
  warn "\n\nNO ENCODING FOUND FOR: $in\n";
  return "UTF-8";
}

sub any2utf {
  my $in = shift;
  my $encoding = guess($in);
  my $out = $converters{$encoding}->convert($in);
  if( $out =~ m/^(Subject|From):/ ){
    if( $out =~ m/^(.*)\=\?(.*?)\?(.*?)\?(.*)\=\?/ ){
      my ($first, $charset, $coding, $second) = ($1,$2,$3,$4);
      if( $coding eq "B" ){ $second = decode_base64($second); }
      elsif ($coding eq "Q" ){ $second = decode_qp($second); }
      else{ warn "Strange line $in\n"; return $out; }
      my $c = Text::Iconv->new($charset, "UTF-8");
      $out = $first . ($c->convert($second));
    }
  }
  return $out;
}

# Le texte en EUC a tendance  contenir des  (hiragana) et des  (katakana)
# Le texte en ISO-2022-JP-2 a tendance  contenir des ESC $ B et ESC ( B (et aussi des ESC pour les accents)
# Le texte en latin1 a tendance  contenir des 
# Le texte en SJIS contient des 0x82 et des 0x83 (hiragana et katakana)
# Le texte en utfcode a tendance  contenir des  (accents) et des couples de 0x8?/0x9?.

